#!/bin/bash

set -e
set -o pipefail

readonly IFS=$'\n\t'
readonly WAIT_TIME_SECS=5
readonly REDIS_MASTER_ADDR="$NODE_IP_ADDR:$NODE_PORT"

kill_server() {
	echo "Killing server processes"
	local redis_pids=($pidof redis-server)
	for redis_pid in ${redis_pids[@]}; do
		kill -15 $redis_pid
	done

	local bash_pids=($pidof bash)
	for bash_pid in ${bash_pids}; do
		kill -15 $bash_pid
	done
}

check_cluster_status() {
	## the following command will only list the redis nodes running on different
	## hosts from the current one
	local all_nodes=$(eval $REDIS_CLI_PATH -p $NODE_PORT $REDIS_PASSWORD_OPTION cluster nodes \
		| grep -v "$NODE_IP_ADDR" \
		| awk -F " " '{print $1,$8}')
	all_nodes=($all_nodes)

	## filter external nodes that are disconnected
	local invalid_nodes=$(echo $all_nodes | grep 'disconnected' || true)
	local invalid_node_count=0
	if [ ! -z "$invalid_nodes" ]; then
		invalid_node_count=$(echo $invalid_nodes | wc -l)
	fi

	if [ "$invalid_node_count" -ne 0 ]; then
		echo "Cluster has invalid nodes"
		for node in ${all_nodes[@]}; do
			local node_id=$(echo $node | awk -F " " '{print $1}')
			local node_status=$(echo $node | awk -F " " '{print $2}')
			if [ "$node_status" == "disconnected" ]; then
				echo "Removing invalid node: $node_id"
				eval $REDIS_CLI_PATH -p $NODE_PORT $REDIS_PASSWORD_OPTION cluster forget $node_id || true
				eval $REDIS_CLI_PATH $REDIS_PASSWORD_OPTION --cluster del-node $REDIS_MASTER_ADDR $node_id || true
			fi
		done
		sleep 5
		echo "All invalid nodes removed, fixing slot mapping"
		eval $REDIS_CLI_PATH $REDIS_PASSWORD_OPTION --cluster fix $REDIS_MASTER_ADDR --cluster-yes
	else
		# All cluster nodes valid, no cleanup required
		true
	fi
}

check_self_status() {
  local redis_server_count=$(pidof redis-server | wc -w)
  if [ $redis_server_count -ne 3 ]; then
    echo "One or more redis-nodes are currently down in the cluster"
    echo "Active node count: $redis_server_count"
    echo "Shutting down the entire cluster"
    exit 1
  fi
}

set_redis_password(){
	REDIS_PASSWORD_OPTION=""
	if [ ! -z "$REDIS_PASSWORD" ]; then
		REDIS_PASSWORD_OPTION="-a \"$REDIS_PASSWORD\" --no-auth-warning"
	fi
}

main() {
  echo "Starting redis cluster cleanup"
  set_redis_password
  while true; do
    check_cluster_status
    check_self_status
    sleep $WAIT_TIME_SECS
  done
}

trap kill_server SIGTERM
trap kill_server SIGSTOP
trap kill_server SIGKILL
trap kill_server EXIT
trap kill_server INT

main
